#!/bin/bash
#
#	check_packages.sh check the necessary packages.
#	check_packages.sh Version: 1.13 (2023/04/11)
#

RPMDIR=""
LISTFILE=""
NGPKGLIST=""
NGFLAG="0"
TEMPDIR="temp"
ARCHLTH=40
PACLTH=50
LOGFILE="$0.log"
ALLSUCCESSFLAG="1"

usage()
{
	echo "Usage: $0 [-i directory]"
	echo "-i directory   Install necessary packages those are not installed."
	echo "               Directory is rpm packages's directory."
	exit 0
}

check_listfile()
{
	if /bin/uname -r 2>/dev/null|/bin/grep "2\\.6\\.32-.*el6" >/dev/null 2>&1 ; then
		MINORVER="`/bin/uname -r 2>/dev/null|/bin/awk 'BEGIN{FS="-"}{print $2}' | awk 'BEGIN{FS="."}{print $1}' 2>/dev/null`"	
		MACHARCH="`/bin/uname -m 2>/dev/null`"
		if [ "$MINORVER" -ge "696" ]; then 
			case "$MACHARCH" in
				i?86)
					LISTFILE="packages/RHEL6.9orLater_x86.txt"
					;;
				x86_64)
					LISTFILE="packages/RHEL6.9orLater_x86_64.txt"
					;;
				*)
					/bin/echo "ERROR: This architecture is not supported. So exit."
					exit 1
					;;
			esac
		else
			case "$MACHARCH" in
				i?86)
					LISTFILE="packages/RHEL6_x86.txt"
					;;
				x86_64)
					LISTFILE="packages/RHEL6_x86_64.txt"
					;;
				*)
					/bin/echo "ERROR: This architecture is not supported. So exit."
					exit 1
					;;
			esac
		fi
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-123\\..*el7" >/dev/null 2>&1 ; then 
		LISTFILE="packages/RHEL7.1orRHEL7.2_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-229\\..*el7" >/dev/null 2>&1 ; then 
		LISTFILE="packages/RHEL7.1orRHEL7.2_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-327\\..*el7" >/dev/null 2>&1 ; then 
		LISTFILE="packages/RHEL7.1orRHEL7.2_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "4\\.1\\.12-.*el7uek" >/dev/null 2>&1 ; then
		LISTFILE="packages/RHEL7.3orRHEL7.4_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-514\\..*el7" >/dev/null 2>&1 ; then
		LISTFILE="packages/RHEL7.3orRHEL7.4_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-693\\..*el7" >/dev/null 2>&1 ; then
		LISTFILE="packages/RHEL7.3orRHEL7.4_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-862\\..*el7" >/dev/null 2>&1 ; then
		LISTFILE="packages/RHEL7.5orRHEL7.6_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-957\\..*el7" >/dev/null 2>&1 ; then
		LISTFILE="packages/RHEL7.5orRHEL7.6_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "3\\.10\\.0-.*el7" >/dev/null 2>&1 ; then 
		LISTFILE="packages/RHEL7_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "4\\.18\\.0-147\\..*el8" >/dev/null 2>&1 ;then
		LISTFILE="packages/RHEL8.1_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "4\\.18\\.0-193\\..*el8" >/dev/null 2>&1 ;then
		LISTFILE="packages/RHEL8.2orRHEL8.3_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "4\\.18\\.0-240\\..*el8" >/dev/null 2>&1 ;then
		LISTFILE="packages/RHEL8.2orRHEL8.3_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "5\\.4\\.17-.*el8uek" >/dev/null 2>&1  ; then 
		LISTFILE="packages/RHEL8.2orRHEL8.3_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "4\\.18\\.0-.*el8" >/dev/null 2>&1  ; then 
		LISTFILE="packages/RHEL8_x86_64.txt"
	elif /bin/uname -r 2>/dev/null|/bin/grep "5\\.14\\.0-.*el9" >/dev/null 2>&1  ; then 
		LISTFILE="packages/RHEL9_x86_64.txt"
	else
		/bin/echo "ERROR: This kernel is not supported. So exit."
		exit 1
	fi
	
	if ! [ -f "${LISTFILE}" ]; then
		/bin/echo "ERROR: The file ${LISTFILE} is not exist. So exit."
		exit 1
	fi
}

install_packages()
{
	cd ${TEMPDIR}
	PERLFLAG=0
	PYTHONFLAG=0
	OPENIPMIFLAG=0
	APRFLAG=0
	for i in `/bin/ls`
	{
		ISPERL=`/bin/echo ${i} | /bin/grep "^perl-"`
		ISPYTHON3=`/bin/echo ${i} | /bin/grep "^python3"`
		ISOPENIPMI=`/bin/echo ${i} | /bin/grep "^OpenIPMI"`
		ISAPR=`/bin/echo ${i} | /bin/grep "^apr"`
		if [ "${ISPERL}" != "" ] ; then
			if [ "${PERLFLAG}" == 0 ] ; then
				/bin/rpm -ivh perl* 2>>../${LOGFILE} 1>/dev/null
				if [ "$?" -ne "0" ] ; then
					/bin/echo "ERROR: Install perl packages failed, please confirm $0.log for detail information."
					ALLSUCCESSFLAG="0"
				fi
				PERLFLAG=1
			fi
		elif [ "${ISPYTHON3}" != "" ] ; then
			if [ "${PYTHONFLAG}" == 0 ] ; then
				/bin/rpm -ivh python3* 2>>../${LOGFILE} 1>/dev/null
				if [ "$?" -ne "0" ] ; then
					/bin/echo "ERROR: Install python3 packages failed, please confirm $0.log for detail information."
					ALLSUCCESSFLAG="0"
				fi
				PYTHONFLAG=1
			fi
		elif [ "${ISOPENIPMI}" != "" ] ; then
			if [ "${OPENIPMIFLAG}" == 0 ] ; then
				/bin/rpm -ivh OpenIPMI* 2>>../${LOGFILE} 1>/dev/null
				if [ "$?" -ne "0" ] ; then
					/bin/echo "ERROR: Install OpenIPMI packages failed, please confirm $0.log for detail information."
					ALLSUCCESSFLAG="0"
				fi
				OPENIPMIFLAG=1
			fi
		elif [ "${ISAPR}" != "" ] ; then
			if [ "${APRFLAG}" == 0 ] ; then
				/bin/rpm -ivh apr* 2>>../${LOGFILE} 1>/dev/null
				if [ "$?" -ne "0" ] ; then
					/bin/echo "ERROR: Install apr packages failed, please confirm $0.log for detail information."
					ALLSUCCESSFLAG="0"
				fi
				APRFLAG=1
			fi
		else
			/bin/rpm -ivh "${i}" 2>>../${LOGFILE} 1>/dev/null
			if [ "$?" -ne "0" ] ; then
				/bin/echo "ERROR: Install ${i} failed, please confirm $0.log for detail information."
				ALLSUCCESSFLAG="0"
			fi
		fi
	}
	cd ..
	/bin/rm -rf ${TEMPDIR} >/dev/null 2>&1
}

deal_packages()
{
	PKGNAME=""
	OPT=""
	ARCHPT=""
	ARCHIS=""
	RESULT=""
	GRP=""
	LASTGRP=""
	NGFLAG="0"
	VERSION=""

	OPT="'%{name}.%{version}.%{arch}'"

	INSTFLAG=$1
	if [ "${INSTFLAG}" == "1" ]; then
		echo -e "\nThe package of [ NG ] will be installed."
	fi

	exec 3< "./$LISTFILE"
	while read GRP PKGNAME ARCHPT VERSION 0<&3
	do
		ARCHIS=`echo ${ARCHPT} | sed -e "s/(\(.*\))/\1/g;s/\\r//g"`
		if [ -z ${VERSION} ] ; then
			RESULT=`rpm -q --qf ${OPT} ${PKGNAME} | grep ${ARCHIS}`
		else
			RESULT=`rpm -q --qf ${OPT} ${PKGNAME} | grep ${ARCHIS} | grep ${VERSION}`
		fi

		if [ "${INSTFLAG}" == "0" ]; then
			if [ "$RESULT" != "" ]; then
				echo -en "${PKGNAME} ${VERSION} \\033[${ARCHLTH}G(${ARCHIS}) \\033[${PACLTH}Gpackage [ OK ]\n"
			else
				echo -en "${PKGNAME} ${VERSION} \\033[${ARCHLTH}G(${ARCHIS}) \\033[${PACLTH}Gpackage [ NG ]\n"
				NGFLAG="2"
			fi
		elif [ "$RESULT" == "" ]; then
			if ! [ -d "${RPMDIR}" ]; then
				echo -e "ERROR: Not found ${RPMDIR} directory."
				exit 1
			fi

			if ! [ "${GRP}" == "${LASTGRP}" ]; then
				if ! [ "${LASTGRP}" == "" ]; then
					install_packages
				fi

				LASTGRP=${GRP}
				/bin/mkdir ${TEMPDIR} >>${LOGFILE} 2>&1
			fi
			
			if [ -z ${VERSION} ] ; then				
				ISEXIST=`/bin/ls "${RPMDIR}" | /bin/grep "^${PKGNAME}-[0-9]" | /bin/grep ${ARCHIS} 2>/dev/null`
			else
				ISEXIST=`/bin/ls "${RPMDIR}" | /bin/grep "^${PKGNAME}-${VERSION}" | /bin/grep ${ARCHIS} 2>/dev/null`
			fi
			if ! [ "${ISEXIST}" == "" ]; then
				/bin/cp -af "${RPMDIR}/${ISEXIST}" ${TEMPDIR} >>${LOGFILE} 2>&1
				if [ "$?" -ne "0" ] ; then
					ALLSUCCESSFLAG="0"
				fi
			else
				ALLSUCCESSFLAG="0"
				echo -e "ERROR: Not found ${RPMDIR}/${PKGNAME}-${VERSION}*.${ARCHIS}.rpm"
			fi
			
 		fi
	done
	exec 3<&-

	if [ "${INSTFLAG}" == "1" ]; then
		install_packages
	fi
}

while [ $# -gt "0" ] ; do
	case $1 in
		"-h")
			usage
			exit 0
			;;
		"-i")
			shift
			if [ $# -gt "0" -a "$1" != "-h" ] ; then
				RPMDIR="$1"
			else
				usage
				exit 1
			fi
			shift
			;;
		*)
			usage
			exit 1
			;;
	esac
done

check_listfile
deal_packages 0

if [ "${NGFLAG}" == "2" ]; then
	if ! [ "${RPMDIR}" == "" ]; then
		if [ -f "${LOGFILE}" ]; then
			/bin/rm -rf ${LOGFILE} >/dev/null 2>&1
		fi
		deal_packages 1
		if [ "${ALLSUCCESSFLAG}" == "1" ]; then
			echo "All packages are installed successfully."
		fi
	else
		echo -e "\nPlease install the package of [ NG ]."
	fi
fi

exit ${NGFLAG}
